@@ -18,7 +18,7 @@ our @EXPORT
     = qw( with template private show show_page attr outs
           outs_raw in_isolation $self under
           get_current_attr xml_decl
-          smart_tag_wrapper current_template );
+          smart_tag_wrapper current_template wrapper );
 our @TAG_SUB_LIST;
 *TagSubs = \@TAG_SUB_LIST;  # For backward compatibility only
 
@@ -101,6 +101,7 @@ Template::Declare::Tags - Build and install XML Tag subroutines for Template::De
             }
         }
         img { attr { src => 'cat.gif' } }
+        img { src is 'dog.gif' }
     };
 
     # Produces:
@@ -111,6 +112,7 @@ Template::Declare::Tags - Build and install XML Tag subroutines for Template::De
     #  </tr>
     # </table>
     # <img src="cat.gif" />
+    # <img src="dog.gif" />
 
     package MyApp::Templates;
 
@@ -160,7 +162,7 @@ You can certainly specify your own tag set classes, as long
 as they subclass L<Template::Declare::TagSet> and implement
 the corresponding methods (e.g. C<get_tag_list>).
 
-If you implement a custome tag set module named
+If you implement a custom tag set module named
 C<Template::Declare::TagSet::Foo>.
 
  use Template::Declare::Tags 'Foo';
@@ -271,6 +273,65 @@ sub template ($$) {
 
 }
 
+=head2 wrapper WRAPPERNAME => sub { 'Implementation' };
+
+C<wrapper> declares a wrapper subroutine that can be called like a tag sub,
+but can optionally take arguments to be passed to the wrapper sub. For
+example, if you wanted to wrap all of the output of a template in the usual
+HTML headers and footers, you can do something like this:
+
+  package MyApp::Templates;
+  use Template::Declare::Tags;
+  use base 'Template::Declare';
+
+  BEGIN {
+      wrapper wrap => sub {
+          my $code = shift;
+          my %params = @_;
+          html {
+              head { title { outs "Hello, $params{user}!"} };
+              body {
+                  $code->();
+                  div { outs 'This is the end, my friend' };
+              };
+          }
+      };
+  }
+
+  template inner => sub {
+      wrap {
+          h1 { outs "Hello, Jesse, s'up?" };
+      } user => 'Jesse';
+  };
+
+Note how the C<wrap> wrapper function is available for calling after it has
+been declared in a C<BEGIN> block. Also note how you can pass arguments to the
+function after the closing brace (you don't need a comma there!).
+
+The output from the "inner" template will look something like this:
+
+  <html>
+   <head>
+    <title>Hello, Jesse!</title>
+   </head>
+   <body>
+    <h1>Hello, Jesse, s&#39;up?</h1>
+    <div>This is the end, my friend</div>
+   </body>
+  </html>
+
+=cut
+
+sub wrapper ($$) {
+    my $wrapper_name   = shift;
+    my $coderef        = shift;
+    my $template_class = caller;
+
+    # Shove the code ref into the calling class.
+    no strict 'refs';
+    *{"$template_class\::$wrapper_name"} = sub (&;@) { goto $coderef };
+}
+
 =head2 private template TEMPLATENAME => sub { 'Implementation' };
 
 C<private> declares that a template isn't available to be called directly from client code.
@@ -291,11 +352,18 @@ With C<attr>, you can specify attributes for HTML tags.
 
 Example:
 
- p { attr { class => 'greeting text',
-            id => 'welcome' };
-
+ p {
+    attr { class => 'greeting text',
+           id    => 'welcome' };
     'This is a welcoming paragraph';
+ }
 
+Tag attributes can also be specified by using C<is>, as in
+
+ p {
+    class is 'greeting text';
+    id    is 'welcome';
+    'This is a welcoming paragraph';
  }
 
 
@@ -537,11 +605,13 @@ sub smart_tag_wrapper (&) {
 
     Template::Declare->new_buffer_frame;
 
+    my %attr = %ATTRIBUTES;
+    %ATTRIBUTES = ();                              # prevent leakage
+
     my $last = join '',    #
         map { ref($_) ? $_ : _postprocess($_) }    #
-        $coderef->(%ATTRIBUTES);
+        $coderef->(%attr);
 
-    %ATTRIBUTES = ();                              # prevent leakage
 
     if ( length( Template::Declare->buffer->data ) ) {
 
@@ -745,7 +815,6 @@ sub _show_template {
     $template = _resolve_relative_template_path($template);
     push @TEMPLATE_STACK, $template;
 
-
     my $callable =
         ( ref($template) && $template->isa('Template::Declare::Tag') )
         ? $template
@@ -759,8 +828,18 @@ sub _show_template {
         return '';
     }
 
+    if (my $instrumentation = Template::Declare->around_template) {
+        $instrumentation->(
+            sub { &$callable($self, @$args) },
+            $template,
+            $args,
+            $callable,
+        );
+    }
+    else {
+        &$callable($self, @$args);
+    }
 
-    &$callable($self, @$args);
     return;
 }
 
@@ -855,7 +934,8 @@ L<Template::Declare::TagSet::XUL>, L<Template::Declare>.
 
 =head1 AUTHOR
 
-Jesse Vincent <jesse@bestpractical.com>
+Jesse Vincent <jesse@bestpractical.com>,
+Agent Zhang <agentzh@yahoo.cn>
 
 =head1 COPYRIGHT
 
